@@ -34,19 +34,24 @@ and port of the master.
 #define WORK_QUEUE_RESULT_OUTPUT_MISSING 16
 #define WORK_QUEUE_RESULT_LINK_FAIL 32
 
-#define WORK_QUEUE_SCHEDULE_UNSET 0 // default setting for task.
+#define WORK_QUEUE_SCHEDULE_UNSET 0	// default setting for task.
 #define WORK_QUEUE_SCHEDULE_FCFS 1
 #define WORK_QUEUE_SCHEDULE_FILES 2
 #define WORK_QUEUE_SCHEDULE_TIME 3
-#define WORK_QUEUE_SCHEDULE_MAX 3
-#define WORK_QUEUE_SCHEDULE_DEFAULT 3 // default setting for queue.
+#define WORK_QUEUE_SCHEDULE_DEFAULT 3	// default setting for queue.
+#define WORK_QUEUE_SCHEDULE_PREFERRED_HOSTS 4
+#define WORK_QUEUE_SCHEDULE_RAND 5
+#define WORK_QUEUE_SCHEDULE_MAX 5
 
 #define WORK_QUEUE_INPUT  0
 #define WORK_QUEUE_OUTPUT 1
 
 #define WORK_QUEUE_NOCACHE 0
 #define WORK_QUEUE_CACHE 1
-
+#define WORK_QUEUE_SYMLINK 2
+#define WORK_QUEUE_PREEXIST 4
+#define WORK_QUEUE_THIRDGET 8
+#define WORK_QUEUE_THIRDPUT 8	// THIRDPUT/THIRDGET identical flags, including both for readability
 
 #define WORK_QUEUE_MASTER_MODE_STANDALONE 0
 #define WORK_QUEUE_MASTER_MODE_CATALOG 1
@@ -59,29 +64,35 @@ and port of the master.
 #define WORK_QUEUE_CATALOG_UPDATE_INTERVAL 60
 #define	WORK_QUEUE_CATALOG_LIFETIME	180
 
+#define WORK_QUEUE_FS_CMD 1
+#define WORK_QUEUE_FS_PATH 2
+#define WORK_QUEUE_FS_SYMLINK 3
+
+
 extern double wq_option_fast_abort_multiplier; /**< Initial setting for fast abort multiplier upon creating queue. Turned off if less than 0. Change prior to calling work_queue_create, after queue is created this variable is not considered and changes must be made through the API calls. */
-extern int wq_option_scheduler; /**< Initial setting for algorithm to assign tasks to workers upon creating queue . Change prior to calling work_queue_create, after queue is created this variable is not considered and changes must be made through the API calls.   */
+extern int wq_option_scheduler;	/**< Initial setting for algorithm to assign tasks to workers upon creating queue . Change prior to calling work_queue_create, after queue is created this variable is not considered and changes must be made through the API calls.   */
 
 /** A task description.  This structure should only be created with @ref work_queue_task_create and delete with @ref work_queue_task_delete.  You may examine (but not modify) this structure once a task has completed.
 */
 struct work_queue_task {
 	char *tag;			/**< An optional user-defined logical name for the task. */
-        char *command_line;		/**< The program(s) to execute, as a shell command line. */
-	int worker_selection_algorithm;           /**< How to choose worker to run the task. */
+	char *command_line;		/**< The program(s) to execute, as a shell command line. */
+	int worker_selection_algorithm;		  /**< How to choose worker to run the task. */
 	char *output;			/**< The standard output of the task. */
-	struct list * input_files;      /**< The files to transfer to the worker and place in the executing directory. */
-	struct list * output_files;	/**< The output files (other than the standard output stream) created by the program expected to be retrieved from the task. */
+	struct list *input_files;	/**< The files to transfer to the worker and place in the executing directory. */
+	struct list *output_files;	/**< The output files (other than the standard output stream) created by the program expected to be retrieved from the task. */
+	char *preferred_host;		/**< The hostname where the task should preferrentially be run. */
 	int taskid;			/**< A unique task id number. */
-    int status;         /**< Current status of the task. */
+	int status;	/**< Current status of the task. */
 	int return_status;		/**< The exit code of the command line. */
 	int result;			/**< The result of the task (successful, failed return_status, missing input file, missing output file). */
-	char* host;			/**< The name of the host on which it ran. */
+	char *host;			/**< The name of the host on which it ran. */
 	timestamp_t submit_time;	/**< The time the task was submitted. */
 	timestamp_t start_time;		/**< The time at which the task began. */
 	timestamp_t finish_time;	/**< The time at which it completed. */
 	timestamp_t transfer_start_time;	/**< The time at which it started to transfer input files. */
-    timestamp_t computation_time;       /**< The time of executing the command. */
-    INT64_T total_bytes_transferred;    /**< Number of bytes transferred since task has last started transferring input data. */
+	timestamp_t computation_time;	/**< The time of executing the command. */
+	INT64_T total_bytes_transferred;/**< Number of bytes transferred since task has last started transferring input data. */
 	timestamp_t total_transfer_time;    /**< Time comsumed in microseconds for transferring total_bytes_transferred. */
 };
 
@@ -99,9 +110,10 @@ struct work_queue_stats {
 	int total_workers_joined;	/**< Total number of times a worker joined the queue. */
 	int total_workers_removed;	/**< Total number of times a worker was removed from the queue. */
 	INT64_T total_bytes_sent;   /**< Total number of file bytes (not including protocol control msg bytes) sent out to the workers by the master. */
-	INT64_T total_bytes_received;   /**< Total number of file bytes (not including protocol control msg bytes) received from the workers by the master. */
-    timestamp_t total_send_time;    /**<Total time in microseconds spent in sending data to workers. */
-    timestamp_t total_receive_time; /**<Total time in microseconds spent in receiving data from workers. */
+	INT64_T total_bytes_received;	/**< Total number of file bytes (not including protocol control msg bytes) received from the workers by the master. */
+	timestamp_t total_send_time;/**<Total time in microseconds spent in sending data to workers. */
+	timestamp_t total_receive_time;
+				    /**<Total time in microseconds spent in receiving data from workers. */
 };
 
 /** @name Functions - Tasks */
@@ -111,19 +123,22 @@ struct work_queue_stats {
 /** Create a new task specification.  Once created, the task may be passed to @ref work_queue_submit.
 @param full_command The shell command line to be executed by the task.
 */
-struct work_queue_task * work_queue_task_create( const char *full_command);
+struct work_queue_task *work_queue_task_create(const char *full_command);
 
 /** Add a file to a task.
 @param t The task to which to add a file.
-@param local_name The name of the file on local disk.
+@param local_name The name of the file on local disk or shared filesystem.
 @param remote_name The name of the file at the execution site.
 @param type Must be one of the following values:
 - WORK_QUEUE_INPUT to indicate an input file to be consumed by the task
 - WORK_QUEUE_OUTPUT to indicate an output file to be produced by the task
 @param flags May be zero to indicate no special handling, or any of the following or'd together:
 - WORK_QUEUE_CACHEABLE - The file may be cached at the execution site for later use.
+- WORK_QUEUE_SYMLINK - Create a symlink to the file rather than copying it, if possible.
+- WORK_QUEUE_THIRDGET - Access the file on the client from a shared filesystem.
+- WORK_QUEUE_THIRDPUT - Access the file on the client from a shared filesystem (included for readability).
 */
-void work_queue_task_specify_file( struct work_queue_task *t, const char *local_name, const char *remote_name, int type, int flags );
+void work_queue_task_specify_file(struct work_queue_task *t, const char *local_name, const char *remote_name, int type, int flags);
 
 /** Add an input buffer to a task.
 @param t The task to which to add a file.
@@ -132,25 +147,43 @@ void work_queue_task_specify_file( struct work_queue_task *t, const char *local_
 @param remote_name The name of the remote file to create.
 @param flags May take the same values as in @ref work_queue_task_specify_file.
 */
-void work_queue_task_specify_buffer( struct work_queue_task *t, const char *data, int length, const char *remote_name, int flags );
+void work_queue_task_specify_buffer(struct work_queue_task *t, const char *data, int length, const char *remote_name, int flags);
+
+/** Add a file created or handled by an arbitrary command to a task (eg: wget, ftp, chirp_get|put).
+@param t The task to which to add a file.
+@param remote_name The name of the file at the execution site.
+@param cmd The command to run on the remote node to retrieve or store the file.
+@param type Must be one of the following values:
+- WORK_QUEUE_INPUT to indicate this creates an input file to be consumed by the task
+- WORK_QUEUE_OUTPUT to indicate this handles an output file to be produced by the task
+@param flags May be zero to indicate no special handling, or any of the following or'd together:
+- WORK_QUEUE_CACHEABLE - The file may be cached at the execution site for later use.
+*/
+void work_queue_task_specify_file_command(struct work_queue_task *t, const char *remote_name, const char *cmd, int type, int flags);
 
 /** Attach a user defined logical name to the task.
 This field is not interpreted by the work queue, but simply maintained to help the user track tasks.
 @param t The task to which to add parameters
 @param tag The tag to attach to task t.
 */
-void work_queue_task_specify_tag( struct work_queue_task *t, const char *tag );
+void work_queue_task_specify_tag(struct work_queue_task *t, const char *tag);
 
 /** Further define a task specification.  Once completed, the task may be passed to @ref work_queue_submit. 
 @param t The task to which to add parameters
 @param alg The algorithm to use in assigning a task to a worker. Valid possibilities are defined in this file as "WORK_QUEUE_SCHEDULE_X" values.
 */
-int work_queue_task_specify_algorithm( struct work_queue_task *t, int alg );
+int work_queue_task_specify_algorithm(struct work_queue_task *t, int alg);
+
+/** Indicate that the task would be optimally run on a given host.
+@param t The task to which to add parameters
+@param hostname The hostname to which this task would optimally be sent.
+*/
+void work_queue_task_specify_preferred_host(struct work_queue_task *t, const char *hostname);
 
 /** Delete a task specification.  This may be called on tasks after they are returned from @ref work_queue_wait.
 @param t The task specification to delete.
 */
-void work_queue_task_delete( struct work_queue_task *t );
+void work_queue_task_delete(struct work_queue_task *t);
 
 //@}
 
@@ -174,69 +207,69 @@ the <b>CATALOG_HOST</b> and <b>CATALOG_PORT</b> environmental variables as descr
 @param port The port number to listen on.  If zero is specified, then the default is chosen, and if -1 is specified, a random port is chosen.  
 @return A new work queue, or null if it could not be created.
 */
-struct work_queue * work_queue_create( int port );
+struct work_queue *work_queue_create(int port);
 
 /** Submit a job to a work queue.
 It is safe to re-submit a task returned by @ref work_queue_wait.
 @param q A work queue returned from @ref work_queue_create.
 @param t A task description returned from @ref work_queue_task_create.
 */
-void work_queue_submit( struct work_queue *q, struct work_queue_task *t );
+void work_queue_submit(struct work_queue *q, struct work_queue_task *t);
 
 /** Wait for tasks to complete.  This call will block until the timeout has elapsed.
 @param q The work queue to wait on.
 @param timeout The number of seconds to wait for a completed task before returning.  Use an integer time to set the timeout or the constant WAITFORTASK to block until a task has completed.
 @returns A completed task description, or null if the queue is empty or the timeout was reached without a completed task.  The returned task must be deleted with @ref work_queue_task_delete or resubmitted with @ref work_queue_submit.
 */
-struct work_queue_task * work_queue_wait( struct work_queue *q, int timeout );
+struct work_queue_task *work_queue_wait(struct work_queue *q, int timeout);
 
 /** Determine whether the queue can support more tasks. Returns the number of additional tasks it can support if "hungry" and 0 if "sated".
 @param q A pointer to the queue to query.
 */
-int work_queue_hungry (struct work_queue *q);
+int work_queue_hungry(struct work_queue *q);
 
 /** Determine whether there are any known tasks queued, running, or waiting to be collected. Returns 0 if there are tasks remaining in the system, 1 if the system is "empty".
 @param q A pointer to the queue to query.
 */
-int work_queue_empty (struct work_queue *q);
+int work_queue_empty(struct work_queue *q);
 
 /** Get the listening port of the queue.
 @param q The work queue of interest.
 @return The port the queue is listening on.
 */
-int work_queue_port( struct work_queue *q );
+int work_queue_port(struct work_queue *q);
 
 /** Get queue statistics.
 @param q The queue to query.
 @param s A pointer to a buffer that will be filed with statistics.
 */
-void work_queue_get_stats( struct work_queue *q, struct work_queue_stats *s );
+void work_queue_get_stats(struct work_queue *q, struct work_queue_stats *s);
 
 /** Turn on or off fast abort functionality for a given queue.
 @param q A pointer to the queue to modify.
 @param multiplier The multiplier of the average task time at which point to abort; if negative (and by default) fast_abort is deactivated.
 @returns 0 if activated or deactivated with an appropriate multiplier, 1 if deactivated due to inappropriate multiplier.
 */
-int work_queue_activate_fast_abort(struct work_queue* q, double multiplier );
+int work_queue_activate_fast_abort(struct work_queue *q, double multiplier);
 
 /** Change the worker selection algorithm for a given queue.
 @param q A pointer to the queue to modify.
 @param alg The algorithm to use in assigning a task to a worker. Valid possibilities are defined in this file as "WORK_QUEUE_SCHEDULE_X" values.
 */
-int work_queue_specify_algorithm( struct work_queue* q, int alg );
+int work_queue_specify_algorithm(struct work_queue *q, int alg);
 
 /** Change the project name for a given queue.
 @param q A pointer to the queue to modify.
 @param name The new project name..
 */
-int work_queue_specify_name( struct work_queue* q, const char *name );
+int work_queue_specify_name(struct work_queue *q, const char *name);
 
 /** Change the priority for a given queue.
 @param q A pointer to the queue to modify.
 @param priority An integer that presents the priorty of this work queue master. The higher the value, the higher the priority.
 @return The priority that has been set.
 */
-int work_queue_specify_priority( struct work_queue* q, int priority );
+int work_queue_specify_priority(struct work_queue *q, int priority);
 
 /** Specify the master mode for a given queue. 
 @param q A pointer to the queue to modify.
@@ -245,7 +278,7 @@ int work_queue_specify_priority( struct work_queue* q, int priority );
 <b>mode == 1</b>: catalog mode. In this mode the master report itself to a catalog server where workers get masters' information and select a master to serve.
 @return The mode that has been set.
 */
-int work_queue_specify_master_mode(struct work_queue* q, int mode);
+int work_queue_specify_master_mode(struct work_queue *q, int mode);
 
 /** Specify the worker mode for a given queue. 
 @param q A pointer to the queue to modify.
@@ -254,18 +287,18 @@ int work_queue_specify_master_mode(struct work_queue* q, int mode);
 <b>mode == 1</b>: exclusive mode. In this mode the master would only accept workers that have specified a preference on it, which are the workers started with "-N name" where name is the name of the queue. 
 @return The mode that has been set.
 */
-int work_queue_specify_worker_mode(struct work_queue* q, int mode);
+int work_queue_specify_worker_mode(struct work_queue *q, int mode);
 
 /** Shut down workers connected to the work_queue system. Gives a best effort and then returns the number of workers given the shut down order.
 @param q A pointer to the queue to query.
 @param n The number to shut down. All workers if given "0".
 */
-int work_queue_shut_down_workers (struct work_queue *q, int n);
+int work_queue_shut_down_workers(struct work_queue *q, int n);
 
 /** Delete a work queue.
 @param q The work queue to delete.
 */
-void work_queue_delete( struct work_queue *q );
+void work_queue_delete(struct work_queue *q);
 
 //@}
 
@@ -280,7 +313,7 @@ void work_queue_delete( struct work_queue *q );
 @param rname The name of the file in which to store the buffer data on the worker
 @deprecated Use @ref work_queue_task_specify_buffer instead.
 */
-void work_queue_task_specify_input_buf( struct work_queue_task *t, const char *buf, int length, const char *rname);
+void work_queue_task_specify_input_buf(struct work_queue_task *t, const char *buf, int length, const char *rname);
 
 /** Add an input file to a task.
 @param t The task to which to add parameters
@@ -288,7 +321,7 @@ void work_queue_task_specify_input_buf( struct work_queue_task *t, const char *b
 @param rname The name of the file in which to store the buffer data on the worker.
 @deprecated See @ref work_queue_task_specify_file instead.
 */
-void work_queue_task_specify_input_file( struct work_queue_task *t, const char *fname, const char *rname);
+void work_queue_task_specify_input_file(struct work_queue_task *t, const char *fname, const char *rname);
 
 /** Add an input file to a task, without caching.
 @param t The task to which to add parameters
@@ -296,7 +329,7 @@ void work_queue_task_specify_input_file( struct work_queue_task *t, const char *
 @param rname The name of the file in which to store the buffer data on the worker.
 @deprecated See @ref work_queue_task_specify_file instead.
 */
-void work_queue_task_specify_input_file_do_not_cache( struct work_queue_task *t, const char *fname, const char *rname);
+void work_queue_task_specify_input_file_do_not_cache(struct work_queue_task *t, const char *fname, const char *rname);
 
 /** Add an output file to a task.
 @param t The task to which to add parameters
@@ -304,7 +337,7 @@ void work_queue_task_specify_input_file_do_not_cache( struct work_queue_task *t,
 @param fname The name of the file local target for copying rname back.
 @deprecated See @ref work_queue_task_specify_file instead.
 */
-void work_queue_task_specify_output_file( struct work_queue_task *t, const char *rname, const char *fname);
+void work_queue_task_specify_output_file(struct work_queue_task *t, const char *rname, const char *fname);
 
 /** Add an output file to a task without caching.
 @param t The task to which to add parameters
@@ -312,7 +345,7 @@ void work_queue_task_specify_output_file( struct work_queue_task *t, const char
 @param fname The name of the file local target for copying rname back.
 @deprecated See @ref work_queue_task_specify_file instead.
 */
-void work_queue_task_specify_output_file_do_not_cache( struct work_queue_task* t, const char* rname, const char* fname);
+void work_queue_task_specify_output_file_do_not_cache(struct work_queue_task *t, const char *rname, const char *fname);
 
 //@}
 
